@@ -126,44 +126,6 @@ def generate_exec_filesystem(protocol_triad, cache_dir, job_data, runtime_root_p
 
     if protocol == 'remote' and not env_and_do_not_replicate:
         check_remote_path(remote_machine, fs_locations["runtime_root"])
-        """
-        if env_instr:
-            tr_cmd = ["bash", data_transfer_protocol, fs_locations["build_root"], "{0}:{1}".format(remote_machine, fs_locations["runtime_root"])]
-            if DEBUG:
-                print("COMMAND", " ".join(tr_cmd))
-                p = subprocess.Popen(tr_cmd)
-            else:
-                p = subprocess.Popen(tr_cmd,
-                    stderr=devnull, stdout=devnull)
-            p.wait()
-        else:
-            mkd1_cmd = ["ssh", remote_machine, "mkdir", fs_locations["runtime_root"]]
-            mkd2_cmd = ["ssh", remote_machine, "mkdir", fs_locations["runtime_exec"]]
-            tr_cmd = ["bash", data_transfer_protocol, fs_locations["build_shared"], "{0}:{1}".format(remote_machine, fs_locations["runtime_shared"])]
-            if DEBUG:
-                print("COMMAND", " ".join(mkd1_cmd))
-                p = subprocess.Popen(mkd1_cmd)
-                p.wait()
-                print("COMMAND", " ".join(mkd2_cmd))
-                p = subprocess.Popen(mkd2_cmd)
-                p.wait()
-                print("COMMAND", " ".join(tr_cmd))
-                p = subprocess.Popen(tr_cmd)
-            else:
-                p = subprocess.Popen(
-                    mkd1_cmd,
-                    stderr=devnull, stdout=devnull)
-                p.wait()
-                p = subprocess.Popen(
-                    mkd2_cmd,
-                    stderr=devnull, stdout=devnull)
-                p.wait()
-                p = subprocess.Popen(
-                    tr_cmd,
-                    stderr=devnull, stdout=devnull)
-            p.wait()
-        time.sleep(10)
-        """
 
     # Step 3: create Work sub-filesystem: "custom" or "locusts" mode -------------------
     if env_instr:  # "custom" mode: environment is created following instruction file
@@ -247,18 +209,6 @@ def generate_exec_filesystem(protocol_triad, cache_dir, job_data, runtime_root_p
                 tf.write(new_command)
             subprocess.call(["chmod", "777", build_task_path])
 
-        # This new cache has to be copied remotely, in a separate way
-        """
-        if protocol == 'remote':
-            tr_cmd = ["bash", data_transfer_protocol, build_cache, "{0}:{1}".format(remote_machine, runtime_cache)]
-            if DEBUG:
-                print("COMMAND", tr_cmd)
-                p = subprocess.Popen(tr_cmd)
-            else:
-                p = subprocess.Popen(tr_cmd,
-                    stderr=devnull, stdout=devnull)
-            p.wait()
-        """
 
     else:  # "locusts" mode: optimized environment for parallel and safe execution
     
@@ -296,18 +246,6 @@ def generate_exec_filesystem(protocol_triad, cache_dir, job_data, runtime_root_p
                     + str(batchno) + '/' + 'task_' + jd['unique_code'] + '/')
             exec_filesystem[jd['unique_code']][0] = (rem_jf, "task.sh")
 
-        """
-        if protocol == "remote":
-            # Transfer local temporary filesystem in the remote location
-            tr_cmd = ["bash", data_transfer_protocol, fs_locations["build_work"], "{0}:{1}".format(remote_machine, fs_locations["runtime_work"])]
-            if DEBUG:
-                print("COMMAND", tr_cmd)
-                p = subprocess.Popen(tr_cmd)
-            else:
-                p = subprocess.Popen(tr_cmd,
-                    stderr=devnull, stdout=devnull)
-            p.wait()
-        """
 
     return {k : exec_filesystem[k][0] for k in exec_filesystem}, fs_locations
 
@@ -468,42 +406,10 @@ def create_manager_scripts(protocol_triad, cache_dir, task_folders, partition,
                 stderr=devnull, stdout=devnull)
         out, err = p.communicate()
 
-    """ 
-    if protocol == 'remote':
-        # Copy all files in exe dir in the remote counterpart
-        filestocopy = fs_locations["build_exec"]+"/"
-        tr_cmd = ["bash", data_transfer_protocol, filestocopy, "{0}:{1}".format(remote_machine, fs_locations["runtime_exec"])]
-        if DEBUG:
-            print("COMMAND", tr_cmd)
-            p = subprocess.Popen(tr_cmd)
-        else:
-            p = subprocess.Popen(tr_cmd,
-                stderr=devnull, stdout=devnull)
-        p.wait()
-    """
-        # Remove the local root directory
-#        p = subprocess.Popen(
-#            ["rm", "-rf", fs_locations["build_root"]],
-#            stderr=devnull, stdout=devnull)
-#        p.wait()
-
-#        rmhidden_cmd = ["ssh", remote_machine, "rm", fs_locations["runtime_exec"]+".*"]
-#    else:
-#        rmhidden_cmd = ["rm", fs_locations["build_exec"]+".*"]
-
-    # It is important to remove any hidden file from the runtime exe dir
-    #  because locusts relies on those files to understand the degree of 
-    #  completion
-#    if DEBUG:
-#        p = subprocess.Popen(rmhidden_cmd)
-#    else:
-#        p = subprocess.Popen(
-#            rmhidden_cmd, stderr=devnull, stdout=devnull
-#        )
-#    p.wait()
+    log_txt = "Actual requested nodes :\t{0}\n".format(requested_nodes)
 
     # List of 2-tuples (job ID, [tasks IDs])
-    return tasks_per_job
+    return tasks_per_job, log_txt
 
 
 def remote_job_control(protocol_triad, batch_job_code, fs_locations, tasks_per_job,
@@ -544,18 +450,25 @@ def remote_job_control(protocol_triad, batch_job_code, fs_locations, tasks_per_j
             p = subprocess.Popen(nhp_cmd,
                 stderr=devnull, stdout=devnull
             )
-#        print(["nohup", fs_locations["runtime_exec"] + mname])
 
     waitcount = {}
     for job_id, task_list in tasks_per_job:
         waitcount[job_id] = 0 
 
     is_over = False
+    chrono_on = {job_id : False for job_id, task_list in tasks_per_job}
+    time_start = {job_id : 0 for job_id, task_list in tasks_per_job}
+    time_end = {job_id : 0 for job_id, task_list in tasks_per_job}
     while not is_over:
         time.sleep(waiting_time)
         is_over = True
         for job_id, task_list in tasks_per_job:
 
+            if chrono_on[job_id] and not time_start[job_id]:
+                time_start[job_id] = time.time()
+            if not chrono_on[job_id] and time_start[job_id]:
+                time_end[job_id] = time.time()
+
             macname = "{0}/.manager_activity_check_{1}{2}".format(
                 fs_locations["runtime_exec"], 
                 batch_job_code, 
@@ -609,8 +522,10 @@ def remote_job_control(protocol_triad, batch_job_code, fs_locations, tasks_per_j
                 is_active = False if coeff > 1 else True
                 if is_active:
                     is_over = False
+                    chrono_on[job_id] = True
                     print("Job", job_id, "running")
                 else:
+                    chrono_on[job_id] = False
                     print("Job", job_id, "ended")
             else:
                 is_there = False
@@ -650,7 +565,27 @@ def remote_job_control(protocol_triad, batch_job_code, fs_locations, tasks_per_j
                 else:
                     print("Job", job_id, "error/aborted")
 
+    log_txt = "\n\n#JOBS LOG\n"
+    time_now = time.time()
+    tot_runtime, totsq = 0, 0
+    for job_id, task_list in tasks_per_job:
+        if not time_end[job_id]: 
+            time_end[job_id] = time_now
+        runtime = time_end[job_id] - time_start[job_id]
+        tot_runtime += runtime
+        totsq += runtime**2
+        log_txt += "JobID {0} :\tRuntime {1}\tRuntime (seconds) {2}\tTasks {3}\n".format(job_id, datetime.timedelta(seconds = int(runtime)), int(runtime), len(task_list))
+
+    av_runtime = tot_runtime/len(tasks_per_job)
+    stdev_runtime = (totsq/len(tasks_per_job) - av_runtime**2)**0.5
+    log_txt += "Total computing time :\t{0}\tSeconds {1}\n".format(datetime.timedelta(seconds = int(tot_runtime)), int(tot_runtime))
+    log_txt += "Average computing time per node:\t{0}\tSeconds {1}\n".format(datetime.timedelta(seconds = int(av_runtime)), int(av_runtime))
+    log_txt += "Time imbalance (std dev):\t{0}\tSeconds {1}\n".format(datetime.timedelta(seconds = int(stdev_runtime)), int(stdev_runtime))
+
     print("Jobs are over")
+    print(log_txt)
+
+    return log_txt
 
 
 def gather_results(protocol_triad, cache_dir, job_data, batch_job_code, 
@@ -890,7 +825,8 @@ def compare_snapshots(snap1, snap2):
             newtosnap2[d] = snap2[d]
             continue
         for f in snap2[d]:
-            if f not in snap1[d] and f not in snap1["::NEGLECT::"]:
+#            print("DECISION", f, f not in snap1[d], f in snap1[d] and snap1[d][f] != snap2[d][f], f not in snap1["::NEGLECT::"])
+            if (f not in snap1[d] or snap1[d][f] != snap2[d][f]) and f not in snap1["::NEGLECT::"]:
                 if d not in newtosnap2:
                     newtosnap2[d] = []
                 newtosnap2[d].append(f)
@@ -1014,6 +950,22 @@ def highly_parallel_job_manager(options, exec_filename,
     partition = options['partition']
     exclusive = options['exclusive']
 
+    log_txt = ("#RUN LOG\n\n"
+        "Run type (local / shared / remote) :\t{0}\n").format(protocol)
+    if protocol != 'local':
+        log_txt += ("Remote machine :\t{0}\n"
+            "Partition (queue) :\t{1}\n"
+            "Number of reserved nodes :\t{2}\n"
+            "Number of cores per node :\t{3}\n"
+            "Exclusive use of nodes :\t{4}\n"
+            "Wall time :\t{5}\n"
+            "Nodewise scratch folder :\t{6}\n"
+            "Nodewise scratch memory :\t{7}\n"
+        ).format(remote_machine, partition, requested_nodes, cpus_per_node, exclusive, wt, nsf, nsm)   
+    else:
+        log_txt += "Number of local processors :\t{0}\n".format(cpus_per_node)
+    print(log_txt)
+
     while job_data:
         # Create the hosting file system
         task_folders, fs_locations = generate_exec_filesystem(
@@ -1028,7 +980,7 @@ def highly_parallel_job_manager(options, exec_filename,
         )
 
         # Create local manager script that does the mpiq job, launchable on each node. It checks the situation regularly each 10 secs.
-        tasks_per_job = create_manager_scripts(
+        tasks_per_job, tmp_log_txt = create_manager_scripts(
             protocol_triad,
             cache_dir, 
             task_folders,
@@ -1046,6 +998,7 @@ def highly_parallel_job_manager(options, exec_filename,
             outer_statements=out_st,
             exclusive=exclusive
         )
+        log_txt += tmp_log_txt
 
         if env_instr:
             snapshot = take_snapshot(
@@ -1067,7 +1020,7 @@ def highly_parallel_job_manager(options, exec_filename,
         # This step is over only when no process is active anymore
         lenlist = [len(x[1]) for x in tasks_per_job]
         waiting_time = min(600, 10*(1+len(job_data)//min(lenlist)))
-        remote_job_control(
+        log_txt += remote_job_control(
             protocol_triad,
             batch_job_code, 
             fs_locations, 
@@ -1095,3 +1048,8 @@ def highly_parallel_job_manager(options, exec_filename,
         if outp:
             for x in outp:
                 output_paths[x] = outp[x]
+
+    run_log_fn = cache_dir + 'run.log'
+    with open(run_log_fn, 'w') as rf:
+        rf.write(log_txt)
+    print('Run log file available at {0}'.format(run_log_fn))
